@@ -3,17 +3,23 @@
 
 """Add a release comment to all the issues mentioned in the latest release."""
 
-import json
 import re
 import sys
 
+from scriv.scriv import Scriv
+
 from session import get_session
 
-with open("tmp/relnotes.json") as frn:
-    relnotes = json.load(frn)
+scriv = Scriv()
+changelog = scriv.changelog()
+changelog.read()
+
+# Get the first entry in the changelog:
+for etitle, sections in changelog.entries().items():
+    version = etitle.split()[1]     # particular to our title format.
+    text = "\n".join(sections)
+    break
 
-latest = relnotes[0]
-version = latest["version"]
 comment = (
     f"This is now released as part of [coverage {version}]" +
     f"(https://pypi.org/project/coverage/{version})."
@@ -21,8 +27,10 @@ comment = (
 print(f"Comment will be:\n\n{comment}\n")
 
 repo_owner = sys.argv[1]
-for m in re.finditer(fr"https://github.com/{repo_owner}/(issues|pull)/(\d+)", latest["text"]):
-    kind, number = m.groups()
+url_matches = re.finditer(fr"https://github.com/{repo_owner}/(issues|pull)/(\d+)", text)
+urls = set((m[0], m[1], m[2]) for m in url_matches)
+
+for url, kind, number in urls:
     do_comment = False
 
     if kind == "issues":
@@ -31,7 +39,7 @@ for m in re.finditer(fr"https://github.com/{repo_owner}/(issues|pull)/(\d+)", la
         if issue_data["state"] == "closed":
             do_comment = True
         else:
-            print(f"Still open, comment manually: {m[0]}")
+            print(f"Still open, comment manually: {url}")
     else:
         url = f"https://api.github.com/repos/{repo_owner}/pulls/{number}"
         pull_data = get_session().get(url).json()
@@ -39,12 +47,12 @@ for m in re.finditer(fr"https://github.com/{repo_owner}/(issues|pull)/(\d+)", la
             if pull_data["merged"]:
                 do_comment = True
             else:
-                print(f"Not merged, comment manually: {m[0]}")
+                print(f"Not merged, comment manually: {url}")
         else:
-            print(f"Still open, comment manually: {m[0]}")
+            print(f"Still open, comment manually: {url}")
 
     if do_comment:
-        print(f"Commenting on {m[0]}")
+        print(f"Commenting on {url}")
         url = f"https://api.github.com/repos/{repo_owner}/issues/{number}/comments"
         resp = get_session().post(url, json={"body": comment})
         print(resp)
