@@ -3,6 +3,8 @@
 
 """Config file for coverage.py"""
 
+from __future__ import annotations
+
 import collections
 import configparser
 import copy
@@ -10,18 +12,26 @@ import os
 import os.path
 import re
 
-from coverage.exceptions import ConfigError
-from coverage.misc import contract, isolate_module, substitute_variables
+from typing import (
+    Any, Callable, Final, Mapping, Union,
+)
+from collections.abc import Iterable
 
+from coverage.exceptions import ConfigError
+from coverage.misc import isolate_module, human_sorted_items, substitute_variables
 from coverage.tomlconfig import TomlConfigParser, TomlDecodeError
+from coverage.types import (
+    TConfigurable, TConfigSectionIn, TConfigValueIn, TConfigSectionOut,
+    TConfigValueOut, TPluginConfig,
+)
 
 os = isolate_module(os)
 
 
-class HandyConfigParser(configparser.RawConfigParser):
+class HandyConfigParser(configparser.ConfigParser):
     """Our specialization of ConfigParser."""
 
-    def __init__(self, our_file):
+    def __init__(self, our_file: bool) -> None:
         """Create the HandyConfigParser.
 
         `our_file` is True if this config file is specifically for coverage,
@@ -29,49 +39,54 @@ class HandyConfigParser(configparser.RawConfigParser):
         for possible settings.
         """
 
-        configparser.RawConfigParser.__init__(self)
+        super().__init__(interpolation=None)
         self.section_prefixes = ["coverage:"]
         if our_file:
             self.section_prefixes.append("")
 
-    def read(self, filenames, encoding_unused=None):
+    def read( # type: ignore[override]
+        self,
+        filenames: Iterable[str],
+        encoding_unused: str | None = None,
+    ) -> list[str]:
         """Read a file name as UTF-8 configuration data."""
-        return configparser.RawConfigParser.read(self, filenames, encoding="utf-8")
+        return super().read(filenames, encoding="utf-8")
 
-    def has_option(self, section, option):
+    def real_section(self, section: str) -> str | None:
+        """Get the actual name of a section."""
         for section_prefix in self.section_prefixes:
             real_section = section_prefix + section
-            has = configparser.RawConfigParser.has_option(self, real_section, option)
-            if has:
-                return has
-        return False
-
-    def has_section(self, section):
-        for section_prefix in self.section_prefixes:
-            real_section = section_prefix + section
-            has = configparser.RawConfigParser.has_section(self, real_section)
+            has = super().has_section(real_section)
             if has:
                 return real_section
+        return None
+
+    def has_option(self, section: str, option: str) -> bool:
+        real_section = self.real_section(section)
+        if real_section is not None:
+            return super().has_option(real_section, option)
         return False
 
-    def options(self, section):
-        for section_prefix in self.section_prefixes:
-            real_section = section_prefix + section
-            if configparser.RawConfigParser.has_section(self, real_section):
-                return configparser.RawConfigParser.options(self, real_section)
+    def has_section(self, section: str) -> bool:
+        return bool(self.real_section(section))
+
+    def options(self, section: str) -> list[str]:
+        real_section = self.real_section(section)
+        if real_section is not None:
+            return super().options(real_section)
         raise ConfigError(f"No section: {section!r}")
 
-    def get_section(self, section):
+    def get_section(self, section: str) -> TConfigSectionOut:
         """Get the contents of a section, as a dictionary."""
-        d = {}
+        d: dict[str, TConfigValueOut] = {}
         for opt in self.options(section):
             d[opt] = self.get(section, opt)
         return d
 
-    def get(self, section, option, *args, **kwargs):
+    def get(self, section: str, option: str, *args: Any, **kwargs: Any) -> str: # type: ignore
         """Get a value, replacing environment variables also.
 
-        The arguments are the same as `RawConfigParser.get`, but in the found
+        The arguments are the same as `ConfigParser.get`, but in the found
         value, ``$WORD`` or ``${WORD}`` are replaced by the value of the
         environment variable ``WORD``.
 
@@ -80,38 +95,38 @@ class HandyConfigParser(configparser.RawConfigParser):
         """
         for section_prefix in self.section_prefixes:
             real_section = section_prefix + section
-            if configparser.RawConfigParser.has_option(self, real_section, option):
+            if super().has_option(real_section, option):
                 break
         else:
             raise ConfigError(f"No option {option!r} in section: {section!r}")
 
-        v = configparser.RawConfigParser.get(self, real_section, option, *args, **kwargs)
+        v: str = super().get(real_section, option, *args, **kwargs)
         v = substitute_variables(v, os.environ)
         return v
 
-    def getlist(self, section, option):
+    def getlist(self, section: str, option: str) -> list[str]:
         """Read a list of strings.
 
         The value of `section` and `option` is treated as a comma- and newline-
-        separated list of strings.  Each value is stripped of whitespace.
+        separated list of strings.  Each value is stripped of white space.
 
         Returns the list of strings.
 
         """
         value_list = self.get(section, option)
         values = []
-        for value_line in value_list.split('\n'):
-            for value in value_line.split(','):
+        for value_line in value_list.split("\n"):
+            for value in value_line.split(","):
                 value = value.strip()
                 if value:
                     values.append(value)
         return values
 
-    def getregexlist(self, section, option):
+    def getregexlist(self, section: str, option: str) -> list[str]:
         """Read a list of full-line regexes.
 
         The value of `section` and `option` is treated as a newline-separated
-        list of regexes.  Each value is stripped of whitespace.
+        list of regexes.  Each value is stripped of white space.
 
         Returns the list of strings.
 
@@ -124,33 +139,36 @@ class HandyConfigParser(configparser.RawConfigParser):
                 re.compile(value)
             except re.error as e:
                 raise ConfigError(
-                    f"Invalid [{section}].{option} value {value!r}: {e}"
+                    f"Invalid [{section}].{option} value {value!r}: {e}",
                 ) from e
             if value:
                 value_list.append(value)
         return value_list
 
 
+TConfigParser = Union[HandyConfigParser, TomlConfigParser]
+
+
 # The default line exclusion regexes.
 DEFAULT_EXCLUDE = [
-    r'#\s*(pragma|PRAGMA)[:\s]?\s*(no|NO)\s*(cover|COVER)',
+    r"#\s*(pragma|PRAGMA)[:\s]?\s*(no|NO)\s*(cover|COVER)",
 ]
 
 # The default partial branch regexes, to be modified by the user.
 DEFAULT_PARTIAL = [
-    r'#\s*(pragma|PRAGMA)[:\s]?\s*(no|NO)\s*(branch|BRANCH)',
+    r"#\s*(pragma|PRAGMA)[:\s]?\s*(no|NO)\s*(branch|BRANCH)",
 ]
 
 # The default partial branch regexes, based on Python semantics.
 # These are any Python branching constructs that can't actually execute all
 # their branches.
 DEFAULT_PARTIAL_ALWAYS = [
-    'while (True|1|False|0):',
-    'if (True|1|False|0):',
+    "while (True|1|False|0):",
+    "if (True|1|False|0):",
 ]
 
 
-class CoverageConfig:
+class CoverageConfig(TConfigurable, TPluginConfig):
     """Coverage.py configuration.
 
     The attributes of this class are the various settings that control the
@@ -159,16 +177,16 @@ class CoverageConfig:
     """
     # pylint: disable=too-many-instance-attributes
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize the configuration attributes to their defaults."""
         # Metadata about the config.
         # We tried to read these config files.
-        self.attempted_config_files = []
+        self.config_files_attempted: list[str] = []
         # We did read these config files, but maybe didn't find any content for us.
-        self.config_files_read = []
+        self.config_files_read: list[str] = []
         # The file that gave us our configuration.
-        self.config_file = None
-        self._config_contents = None
+        self.config_file: str | None = None
+        self._config_contents: bytes | None = None
 
         # Defaults for [run] and [report]
         self._include = None
@@ -176,45 +194,50 @@ class CoverageConfig:
 
         # Defaults for [run]
         self.branch = False
-        self.command_line = None
-        self.concurrency = None
-        self.context = None
+        self.command_line: str | None = None
+        self.concurrency: list[str] = []
+        self.context: str | None = None
         self.cover_pylib = False
         self.data_file = ".coverage"
-        self.debug = []
-        self.disable_warnings = []
-        self.dynamic_context = None
-        self.note = None
+        self.debug: list[str] = []
+        self.debug_file: str | None = None
+        self.disable_warnings: list[str] = []
+        self.dynamic_context: str | None = None
         self.parallel = False
-        self.plugins = []
+        self.plugins: list[str] = []
         self.relative_files = False
-        self.run_include = None
-        self.run_omit = None
-        self.source = None
-        self.source_pkgs = []
+        self.run_include: list[str] = []
+        self.run_omit: list[str] = []
+        self.sigterm = False
+        self.source: list[str] | None = None
+        self.source_pkgs: list[str] = []
+        self.source_dirs: list[str] = []
         self.timid = False
-        self._crash = None
+        self._crash: str | None = None
 
         # Defaults for [report]
         self.exclude_list = DEFAULT_EXCLUDE[:]
+        self.exclude_also: list[str] = []
         self.fail_under = 0.0
+        self.format: str | None = None
         self.ignore_errors = False
-        self.report_include = None
-        self.report_omit = None
+        self.include_namespace_packages = False
+        self.report_include: list[str] | None = None
+        self.report_omit: list[str] | None = None
         self.partial_always_list = DEFAULT_PARTIAL_ALWAYS[:]
         self.partial_list = DEFAULT_PARTIAL[:]
         self.precision = 0
-        self.report_contexts = None
+        self.report_contexts: list[str] | None = None
         self.show_missing = False
         self.skip_covered = False
         self.skip_empty = False
-        self.sort = None
+        self.sort: str | None = None
 
         # Defaults for [html]
-        self.extra_css = None
+        self.extra_css: str | None = None
         self.html_dir = "htmlcov"
-        self.html_skip_covered = None
-        self.html_skip_empty = None
+        self.html_skip_covered: bool | None = None
+        self.html_skip_empty: bool | None = None
         self.html_title = "Coverage report"
         self.show_contexts = False
 
@@ -227,11 +250,15 @@ class CoverageConfig:
         self.json_pretty_print = False
         self.json_show_contexts = False
 
+        # Defaults for [lcov]
+        self.lcov_output = "coverage.lcov"
+        self.lcov_line_checksums = False
+
         # Defaults for [paths]
-        self.paths = collections.OrderedDict()
+        self.paths: dict[str, list[str]] = {}
 
         # Options for plugins
-        self.plugin_options = {}
+        self.plugin_options: dict[str, TConfigSectionOut] = {}
 
     MUST_BE_LIST = {
         "debug", "concurrency", "plugins",
@@ -239,7 +266,7 @@ class CoverageConfig:
         "run_omit", "run_include",
     }
 
-    def from_args(self, **kwargs):
+    def from_args(self, **kwargs: TConfigValueIn) -> None:
         """Read config values from `kwargs`."""
         for k, v in kwargs.items():
             if v is not None:
@@ -247,8 +274,7 @@ class CoverageConfig:
                     v = [v]
                 setattr(self, k, v)
 
-    @contract(filename=str)
-    def from_file(self, filename, warn, our_file):
+    def from_file(self, filename: str, warn: Callable[[str], None], our_file: bool) -> bool:
         """Read configuration from a .rc file.
 
         `filename` is a file name to read.
@@ -262,12 +288,13 @@ class CoverageConfig:
 
         """
         _, ext = os.path.splitext(filename)
-        if ext == '.toml':
+        cp: TConfigParser
+        if ext == ".toml":
             cp = TomlConfigParser(our_file)
         else:
             cp = HandyConfigParser(our_file)
 
-        self.attempted_config_files.append(filename)
+        self.config_files_attempted.append(os.path.abspath(filename))
 
         try:
             files_read = cp.read(filename)
@@ -294,19 +321,19 @@ class CoverageConfig:
             all_options[section].add(option)
 
         for section, options in all_options.items():
-            real_section = cp.has_section(section)
+            real_section = cp.real_section(section)
             if real_section:
                 for unknown in set(cp.options(section)) - options:
                     warn(
                         "Unrecognized option '[{}] {}=' in config file {}".format(
-                            real_section, unknown, filename
-                        )
+                            real_section, unknown, filename,
+                        ),
                     )
 
         # [paths] is special
-        if cp.has_section('paths'):
-            for option in cp.options('paths'):
-                self.paths[option] = cp.getlist('paths', option)
+        if cp.has_section("paths"):
+            for option in cp.options("paths"):
+                self.paths[option] = cp.getlist("paths", option)
                 any_set = True
 
         # plugins can have options
@@ -330,11 +357,13 @@ class CoverageConfig:
 
         return used
 
-    def copy(self):
+    def copy(self) -> CoverageConfig:
         """Return a copy of the configuration."""
         return copy.deepcopy(self)
 
-    CONCURRENCY_CHOICES = {"thread", "gevent", "greenlet", "eventlet", "multiprocessing"}
+    CONCURRENCY_CHOICES: Final[set[str]] = {
+        "thread", "gevent", "greenlet", "eventlet", "multiprocessing"
+    }
 
     CONFIG_FILE_OPTIONS = [
         # These are *args for _set_attr_from_config_option:
@@ -346,60 +375,75 @@ class CoverageConfig:
         #       configuration value from the file.
 
         # [run]
-        ('branch', 'run:branch', 'boolean'),
-        ('command_line', 'run:command_line'),
-        ('concurrency', 'run:concurrency', 'list'),
-        ('context', 'run:context'),
-        ('cover_pylib', 'run:cover_pylib', 'boolean'),
-        ('data_file', 'run:data_file'),
-        ('debug', 'run:debug', 'list'),
-        ('disable_warnings', 'run:disable_warnings', 'list'),
-        ('dynamic_context', 'run:dynamic_context'),
-        ('note', 'run:note'),
-        ('parallel', 'run:parallel', 'boolean'),
-        ('plugins', 'run:plugins', 'list'),
-        ('relative_files', 'run:relative_files', 'boolean'),
-        ('run_include', 'run:include', 'list'),
-        ('run_omit', 'run:omit', 'list'),
-        ('source', 'run:source', 'list'),
-        ('source_pkgs', 'run:source_pkgs', 'list'),
-        ('timid', 'run:timid', 'boolean'),
-        ('_crash', 'run:_crash'),
+        ("branch", "run:branch", "boolean"),
+        ("command_line", "run:command_line"),
+        ("concurrency", "run:concurrency", "list"),
+        ("context", "run:context"),
+        ("cover_pylib", "run:cover_pylib", "boolean"),
+        ("data_file", "run:data_file"),
+        ("debug", "run:debug", "list"),
+        ("debug_file", "run:debug_file"),
+        ("disable_warnings", "run:disable_warnings", "list"),
+        ("dynamic_context", "run:dynamic_context"),
+        ("parallel", "run:parallel", "boolean"),
+        ("plugins", "run:plugins", "list"),
+        ("relative_files", "run:relative_files", "boolean"),
+        ("run_include", "run:include", "list"),
+        ("run_omit", "run:omit", "list"),
+        ("sigterm", "run:sigterm", "boolean"),
+        ("source", "run:source", "list"),
+        ("source_pkgs", "run:source_pkgs", "list"),
+        ("source_dirs", "run:source_dirs", "list"),
+        ("timid", "run:timid", "boolean"),
+        ("_crash", "run:_crash"),
 
         # [report]
-        ('exclude_list', 'report:exclude_lines', 'regexlist'),
-        ('fail_under', 'report:fail_under', 'float'),
-        ('ignore_errors', 'report:ignore_errors', 'boolean'),
-        ('partial_always_list', 'report:partial_branches_always', 'regexlist'),
-        ('partial_list', 'report:partial_branches', 'regexlist'),
-        ('precision', 'report:precision', 'int'),
-        ('report_contexts', 'report:contexts', 'list'),
-        ('report_include', 'report:include', 'list'),
-        ('report_omit', 'report:omit', 'list'),
-        ('show_missing', 'report:show_missing', 'boolean'),
-        ('skip_covered', 'report:skip_covered', 'boolean'),
-        ('skip_empty', 'report:skip_empty', 'boolean'),
-        ('sort', 'report:sort'),
+        ("exclude_list", "report:exclude_lines", "regexlist"),
+        ("exclude_also", "report:exclude_also", "regexlist"),
+        ("fail_under", "report:fail_under", "float"),
+        ("format", "report:format"),
+        ("ignore_errors", "report:ignore_errors", "boolean"),
+        ("include_namespace_packages", "report:include_namespace_packages", "boolean"),
+        ("partial_always_list", "report:partial_branches_always", "regexlist"),
+        ("partial_list", "report:partial_branches", "regexlist"),
+        ("precision", "report:precision", "int"),
+        ("report_contexts", "report:contexts", "list"),
+        ("report_include", "report:include", "list"),
+        ("report_omit", "report:omit", "list"),
+        ("show_missing", "report:show_missing", "boolean"),
+        ("skip_covered", "report:skip_covered", "boolean"),
+        ("skip_empty", "report:skip_empty", "boolean"),
+        ("sort", "report:sort"),
 
         # [html]
-        ('extra_css', 'html:extra_css'),
-        ('html_dir', 'html:directory'),
-        ('html_skip_covered', 'html:skip_covered', 'boolean'),
-        ('html_skip_empty', 'html:skip_empty', 'boolean'),
-        ('html_title', 'html:title'),
-        ('show_contexts', 'html:show_contexts', 'boolean'),
+        ("extra_css", "html:extra_css"),
+        ("html_dir", "html:directory"),
+        ("html_skip_covered", "html:skip_covered", "boolean"),
+        ("html_skip_empty", "html:skip_empty", "boolean"),
+        ("html_title", "html:title"),
+        ("show_contexts", "html:show_contexts", "boolean"),
 
         # [xml]
-        ('xml_output', 'xml:output'),
-        ('xml_package_depth', 'xml:package_depth', 'int'),
+        ("xml_output", "xml:output"),
+        ("xml_package_depth", "xml:package_depth", "int"),
 
         # [json]
-        ('json_output', 'json:output'),
-        ('json_pretty_print', 'json:pretty_print', 'boolean'),
-        ('json_show_contexts', 'json:show_contexts', 'boolean'),
+        ("json_output", "json:output"),
+        ("json_pretty_print", "json:pretty_print", "boolean"),
+        ("json_show_contexts", "json:show_contexts", "boolean"),
+
+        # [lcov]
+        ("lcov_output", "lcov:output"),
+        ("lcov_line_checksums", "lcov:line_checksums", "boolean")
     ]
 
-    def _set_attr_from_config_option(self, cp, attr, where, type_=''):
+    def _set_attr_from_config_option(
+        self,
+        cp: TConfigParser,
+        attr: str,
+        where: str,
+        type_: str = "",
+    ) -> bool:
         """Set an attribute on self if it exists in the ConfigParser.
 
         Returns True if the attribute was set.
@@ -407,16 +451,16 @@ class CoverageConfig:
         """
         section, option = where.split(":")
         if cp.has_option(section, option):
-            method = getattr(cp, 'get' + type_)
+            method = getattr(cp, f"get{type_}")
             setattr(self, attr, method(section, option))
             return True
         return False
 
-    def get_plugin_options(self, plugin):
+    def get_plugin_options(self, plugin: str) -> TConfigSectionOut:
         """Get a dictionary of options for the plugin named `plugin`."""
         return self.plugin_options.get(plugin, {})
 
-    def set_option(self, option_name, value):
+    def set_option(self, option_name: str, value: TConfigValueIn | TConfigSectionIn) -> None:
         """Set an option in the configuration.
 
         `option_name` is a colon-separated string indicating the section and
@@ -428,7 +472,13 @@ class CoverageConfig:
         """
         # Special-cased options.
         if option_name == "paths":
-            self.paths = value
+            # This is ugly, but type-checks and ensures the values are close
+            # to right.
+            self.paths = {}
+            assert isinstance(value, Mapping)
+            for k, v in value.items():
+                assert isinstance(v, Iterable)
+                self.paths[k] = list(v)
             return
 
         # Check all the hard-coded options.
@@ -441,13 +491,13 @@ class CoverageConfig:
         # See if it's a plugin option.
         plugin_name, _, key = option_name.partition(":")
         if key and plugin_name in self.plugins:
-            self.plugin_options.setdefault(plugin_name, {})[key] = value
+            self.plugin_options.setdefault(plugin_name, {})[key] = value # type: ignore[index]
             return
 
         # If we get here, we didn't find the option.
         raise ConfigError(f"No such option: {option_name!r}")
 
-    def get_option(self, option_name):
+    def get_option(self, option_name: str) -> TConfigValueOut | None:
         """Get an option from the configuration.
 
         `option_name` is a colon-separated string indicating the section and
@@ -465,7 +515,7 @@ class CoverageConfig:
         for option_spec in self.CONFIG_FILE_OPTIONS:
             attr, where = option_spec[:2]
             if where == option_name:
-                return getattr(self, attr)
+                return getattr(self, attr)  # type: ignore[no-any-return]
 
         # See if it's a plugin option.
         plugin_name, _, key = option_name.partition(":")
@@ -475,22 +525,29 @@ class CoverageConfig:
         # If we get here, we didn't find the option.
         raise ConfigError(f"No such option: {option_name!r}")
 
-    def post_process_file(self, path):
+    def post_process_file(self, path: str) -> str:
         """Make final adjustments to a file path to make it usable."""
         return os.path.expanduser(path)
 
-    def post_process(self):
+    def post_process(self) -> None:
         """Make final adjustments to settings to make them usable."""
         self.data_file = self.post_process_file(self.data_file)
         self.html_dir = self.post_process_file(self.html_dir)
         self.xml_output = self.post_process_file(self.xml_output)
-        self.paths = collections.OrderedDict(
-            (k, [self.post_process_file(f) for f in v])
+        self.paths = {
+            k: [self.post_process_file(f) for f in v]
             for k, v in self.paths.items()
+        }
+        self.exclude_list += self.exclude_also
+
+    def debug_info(self) -> list[tuple[str, Any]]:
+        """Make a list of (name, value) pairs for writing debug info."""
+        return human_sorted_items(
+            (k, v) for k, v in self.__dict__.items() if not k.startswith("_")
         )
 
 
-def config_files_to_try(config_file):
+def config_files_to_try(config_file: bool | str) -> list[tuple[str, bool, bool]]:
     """What config files should we try to read?
 
     Returns a list of tuples:
@@ -504,12 +561,14 @@ def config_files_to_try(config_file):
     specified_file = (config_file is not True)
     if not specified_file:
         # No file was specified. Check COVERAGE_RCFILE.
-        config_file = os.environ.get('COVERAGE_RCFILE')
-        if config_file:
+        rcfile = os.getenv("COVERAGE_RCFILE")
+        if rcfile:
+            config_file = rcfile
             specified_file = True
     if not specified_file:
         # Still no file specified. Default to .coveragerc
         config_file = ".coveragerc"
+    assert isinstance(config_file, str)
     files_to_try = [
         (config_file, True, specified_file),
         ("setup.cfg", False, False),
@@ -519,7 +578,11 @@ def config_files_to_try(config_file):
     return files_to_try
 
 
-def read_coverage_config(config_file, warn, **kwargs):
+def read_coverage_config(
+    config_file: bool | str,
+    warn: Callable[[str], None],
+    **kwargs: TConfigValueIn,
+) -> CoverageConfig:
     """Read the coverage.py configuration.
 
     Arguments:
@@ -550,18 +613,23 @@ def read_coverage_config(config_file, warn, **kwargs):
             if specified_file:
                 raise ConfigError(f"Couldn't read {fname!r} as a config file")
 
-    # $set_env.py: COVERAGE_DEBUG - Options for --debug.
     # 3) from environment variables:
-    env_data_file = os.environ.get('COVERAGE_FILE')
+    env_data_file = os.getenv("COVERAGE_FILE")
     if env_data_file:
         config.data_file = env_data_file
-    debugs = os.environ.get('COVERAGE_DEBUG')
+    # $set_env.py: COVERAGE_DEBUG - Debug options: https://coverage.rtfd.io/cmd.html#debug
+    debugs = os.getenv("COVERAGE_DEBUG")
     if debugs:
         config.debug.extend(d.strip() for d in debugs.split(","))
 
     # 4) from constructor arguments:
     config.from_args(**kwargs)
 
+    # 5) for our benchmark, force settings using a secret environment variable:
+    force_file = os.getenv("COVERAGE_FORCE_CONFIG")
+    if force_file:
+        config.from_file(force_file, warn, our_file=True)
+
     # Once all the config has been collected, there's a little post-processing
     # to do.
     config.post_process()
