@@ -41,13 +41,6 @@
 #define MyFrame_SetTrace(f, obj)    {Py_INCREF(obj); Py_XSETREF((f)->f_trace, (PyObject*)(obj));}
 #endif
 
-// Access f_code should be done through a helper starting in 3.9.
-#if PY_VERSION_HEX >= 0x03090000
-#define MyFrame_GetCode(f)      (PyFrame_GetCode(f))
-#else
-#define MyFrame_GetCode(f)      ((f)->f_code)
-#endif
-
 #if PY_VERSION_HEX >= 0x030B00B1
 #define MyCode_GetCode(co)      (PyCode_GetCode(co))
 #define MyCode_FreeCode(code)   Py_XDECREF(code)
@@ -60,9 +53,10 @@
 #endif
 
 // Where does frame.f_lasti point when yielding from a generator?
-// It used to point at the YIELD, now it points at the RESUME.
+// It used to point at the YIELD, in 3.13 it points at the RESUME,
+// then it went back to the YIELD.
 // https://github.com/python/cpython/issues/113728
-#define ENV_LASTI_IS_YIELD (PY_VERSION_HEX < 0x030D0000)
+#define ENV_LASTI_IS_YIELD ((PY_VERSION_HEX & 0xFFFF0000) != 0x030D0000)
 
 /* The values returned to indicate ok or error. */
 #define RET_OK      0
