@@ -15,7 +15,8 @@ import re
 import string
 
 from dataclasses import dataclass, field
-from typing import Any, Iterable, TYPE_CHECKING
+from typing import Any, TYPE_CHECKING
+from collections.abc import Iterable
 
 import coverage
 from coverage.data import CoverageData, add_data_to_hash
@@ -50,7 +51,7 @@ def data_filename(fname: str) -> str:
 
 def read_data(fname: str) -> str:
     """Return the contents of a data file of ours."""
-    with open(data_filename(fname)) as data_file:
+    with open(data_filename(fname), encoding="utf-8") as data_file:
         return data_file.read()
 
 
@@ -136,6 +137,7 @@ class HtmlDataGeneration:
             contexts_by_lineno = self.data.contexts_by_lineno(analysis.filename)
 
         lines = []
+        branch_stats = analysis.branch_stats()
 
         for lineno, tokens in enumerate(fr.source_token_lines(), start=1):
             # Figure out how to mark this line.
@@ -149,12 +151,22 @@ class HtmlDataGeneration:
                 category = "mis"
             elif self.has_arcs and lineno in missing_branch_arcs:
                 category = "par"
-                for b in missing_branch_arcs[lineno]:
-                    if b < 0:
-                        short_annotations.append("exit")
-                    else:
-                        short_annotations.append(str(b))
-                    long_annotations.append(fr.missing_arc_description(lineno, b, arcs_executed))
+                mba = missing_branch_arcs[lineno]
+                if len(mba) == branch_stats[lineno][0]:
+                    # None of the branches were taken from this line.
+                    short_annotations.append("anywhere")
+                    long_annotations.append(
+                        f"line {lineno} didn't jump anywhere: it always raised an exception."
+                    )
+                else:
+                    for b in missing_branch_arcs[lineno]:
+                        if b < 0:
+                            short_annotations.append("exit")
+                        else:
+                            short_annotations.append(str(b))
+                        long_annotations.append(
+                            fr.missing_arc_description(lineno, b, arcs_executed)
+                        )
             elif lineno in analysis.statements:
                 category = "run"
 
@@ -201,7 +213,7 @@ class FileToReport:
 
 HTML_SAFE = string.ascii_letters + string.digits + "!#$%'()*+,-./:;=?@[]^_`{|}~"
 
-@functools.lru_cache(maxsize=None)
+@functools.cache
 def encode_int(n: int) -> str:
     """Create a short HTML-safe string from an integer, using HTML_SAFE."""
     if n == 0:
@@ -400,7 +412,7 @@ class HtmlReporter:
         # .gitignore can't be copied from the source tree because if it was in
         # the source tree, it would stop the static files from being checked in.
         if self.directory_was_empty:
-            with open(os.path.join(self.directory, ".gitignore"), "w") as fgi:
+            with open(os.path.join(self.directory, ".gitignore"), "w", encoding="utf-8") as fgi:
                 fgi.write("# Created by coverage.py\n*\n")
 
     def should_report(self, analysis: Analysis, index_page: IndexPage) -> bool:
@@ -485,16 +497,12 @@ class HtmlReporter:
 
             if ldata.long_annotations:
                 longs = ldata.long_annotations
-                if len(longs) == 1:
-                    ldata.annotate_long = longs[0]
-                else:
-                    ldata.annotate_long = "{:d} missed branches: {}".format(
-                        len(longs),
-                        ", ".join(
-                            f"{num:d}) {ann_long}"
-                            for num, ann_long in enumerate(longs, start=1)
-                        ),
-                    )
+                # A line can only have two branch destinations. If there were
+                # two missing, we would have written one as "always raised."
+                assert len(longs) == 1, (
+                    f"Had long annotations in {ftr.fr.relative_filename()}: {longs}"
+                )
+                ldata.annotate_long = longs[0]
             else:
                 ldata.annotate_long = None
 
@@ -698,7 +706,7 @@ class IncrementalChecker:
         """Read the information we stored last time."""
         try:
             status_file = os.path.join(self.directory, self.STATUS_FILE)
-            with open(status_file) as fstatus:
+            with open(status_file, encoding="utf-8") as fstatus:
                 status = json.load(fstatus)
         except (OSError, ValueError):
             # Status file is missing or malformed.
@@ -739,7 +747,7 @@ class IncrementalChecker:
                 for fname, finfo in self.files.items()
             },
         }
-        with open(status_file, "w") as fout:
+        with open(status_file, "w", encoding="utf-8") as fout:
             json.dump(status_data, fout, separators=(",", ":"))
 
     def check_global_data(self, *data: Any) -> None:
