@@ -15,25 +15,27 @@ import os.path
 import re
 import shutil
 import subprocess
+import sys
 import textwrap
 import warnings
 
 from pathlib import Path
 from typing import (
-    Any, Callable, Iterable, Iterator, NoReturn, TypeVar, cast,
+    Any, Callable, NoReturn, TypeVar, cast,
 )
+from collections.abc import Iterable, Iterator
 
 import flaky
-import pytest
 
+import coverage
 from coverage import env
 from coverage.debug import DebugControl
 from coverage.exceptions import CoverageWarning
-from coverage.types import TArc, TLineNo
+from coverage.types import TArc
 
 
 def run_command(cmd: str) -> tuple[int, str]:
-    """Run a command in a sub-process.
+    """Run a command in a subprocess.
 
     Returns the exit status code and the combined stdout and stderr.
 
@@ -41,10 +43,18 @@ def run_command(cmd: str) -> tuple[int, str]:
     # Subprocesses are expensive, but convenient, and so may be over-used in
     # the test suite.  Use these lines to get a list of the tests using them:
     if 0:  # pragma: debugging
-        with open("/tmp/processes.txt", "a") as proctxt:  # type: ignore[unreachable]
+        pth = "/tmp/processes.txt"                      # type: ignore[unreachable]
+        with open(pth, "a", encoding="utf-8") as proctxt:
             print(os.getenv("PYTEST_CURRENT_TEST", "unknown"), file=proctxt, flush=True)
 
-    encoding = os.device_encoding(1) or locale.getpreferredencoding()
+    # Type checking trick due to "unreachable" being set
+    _locale_type_erased: Any = locale
+
+    encoding = os.device_encoding(1) or (
+        _locale_type_erased.getpreferredencoding()
+        if sys.version_info < (3, 11)
+        else _locale_type_erased.getencoding()
+    )
 
     # In some strange cases (PyPy3 in a virtualenv!?) the stdout encoding of
     # the subprocess is set incorrectly to ascii.  Use an environment variable
@@ -112,7 +122,7 @@ def make_file(
 
     if text and basename.endswith(".py") and SHOW_DIS:      # pragma: debugging
         os.makedirs("/tmp/dis", exist_ok=True)
-        with open(f"/tmp/dis/{basename}.dis", "w") as fdis:
+        with open(f"/tmp/dis/{basename}.dis", "w", encoding="utf-8") as fdis:
             print(f"# {os.path.abspath(filename)}", file=fdis)
             cur_test = os.getenv("PYTEST_CURRENT_TEST", "unknown")
             print(f"# PYTEST_CURRENT_TEST = {cur_test}", file=fdis)
@@ -259,38 +269,6 @@ def arcz_to_arcs(arcz: str) -> list[TArc]:
     return sorted(arcs)
 
 
-_arcz_unmap = {val: ch for ch, val in _arcz_map.items()}
-
-
-def _arcs_to_arcz_repr_one(num: TLineNo) -> str:
-    """Return an arcz form of the number `num`, or "?" if there is none."""
-    if num == -1:
-        return "."
-    z = ""
-    if num < 0:
-        z += "-"
-        num *= -1
-    z += _arcz_unmap.get(num, "?")
-    return z
-
-
-def arcs_to_arcz_repr(arcs: Iterable[TArc] | None) -> str:
-    """Convert a list of arcs to a readable multi-line form for asserting.
-
-    Each pair is on its own line, with a comment showing the arcz form,
-    to make it easier to decode when debugging test failures.
-
-    """
-    repr_list = []
-    for a, b in (arcs or ()):
-        line = repr((a, b))
-        line += " # "
-        line += _arcs_to_arcz_repr_one(a)
-        line += _arcs_to_arcz_repr_one(b)
-        repr_list.append(line)
-    return "\n".join(repr_list) + "\n"
-
-
 @contextlib.contextmanager
 def change_dir(new_dir: str | Path) -> Iterator[None]:
     """Change directory, and then change back.
@@ -358,12 +336,6 @@ def swallow_warnings(
         yield
 
 
-xfail_pypy38 = pytest.mark.xfail(
-    env.PYPY and env.PYVERSION[:2] == (3, 8) and env.PYPYVERSION < (7, 3, 11),
-    reason="These tests fail on older PyPy 3.8",
-)
-
-
 class FailingProxy:
     """A proxy for another object, but one method will fail a few times before working."""
     def __init__(self, obj: Any, methname: str, fails: list[Exception]) -> None:
@@ -411,3 +383,26 @@ def flaky_method(max_runs: int) -> Callable[[TestMethod], TestMethod]:
     def _decorator(fn: TestMethod) -> TestMethod:
         return cast(TestMethod, flaky.flaky(max_runs)(fn))
     return _decorator
+
+
+def all_our_source_files() -> Iterator[tuple[Path, str]]:
+    """Iterate over all of our own source files.
+
+    This is used in tests that need a bunch of Python code to analyze, so we
+    might as well use our own source code as the subject.
+
+    Produces a stream of (filename, file contents) tuples.
+    """
+    print(f"all_our_source_files: {coverage.__file__ = }")
+    cov_dir = Path(coverage.__file__).parent.parent
+    if ".tox" in cov_dir.parts:
+        # We are in a tox-installed environment, look above the .tox dir to
+        # also find the uninstalled source files.
+        cov_dir = Path(os.fspath(cov_dir).partition(".tox")[0])
+
+    print(f"all_our_source_files: {os.path.abspath(cov_dir) = }")
+    # To run against all the files in the tox venvs:
+    #   for source_file in cov_dir.rglob("*.py"):
+    for sub in [".", "benchmark", "ci", "coverage", "lab", "tests"]:
+        for source_file in (cov_dir / sub).glob("*.py"):
+            yield (source_file, source_file.read_text(encoding="utf-8"))
