@@ -13,7 +13,8 @@ import sys
 import textwrap
 
 from unittest import mock
-from typing import Any, Mapping
+from typing import Any
+from collections.abc import Mapping
 
 import pytest
 
