@@ -313,7 +313,11 @@ class ConfigTest(CoverageTest):
             s = s.replace("~joe/", "/Users/joe/")
             return s
 
-        with mock.patch.object(coverage.config.os.path, 'expanduser', new=expanduser):
+        with mock.patch.object(
+            coverage.config.os.path,  # type: ignore[attr-defined]
+            'expanduser',
+            new=expanduser
+        ):
             cov = coverage.Coverage()
         assert cov.config.data_file == "/Users/me/data.file"
         assert cov.config.html_dir == "/Users/joe/html_dir"
@@ -352,7 +356,11 @@ class ConfigTest(CoverageTest):
             s = s.replace("~joe/", "/Users/joe/")
             return s
 
-        with mock.patch.object(coverage.config.os.path, 'expanduser', new=expanduser):
+        with mock.patch.object(
+            coverage.config.os.path, # type: ignore[attr-defined]
+            'expanduser',
+            new=expanduser
+        ):
             cov = coverage.Coverage()
         assert cov.config.data_file == "/Users/me/data.file"
         assert cov.config.html_dir == "/Users/joe/html_dir"
@@ -502,6 +510,7 @@ class ConfigFileTest(UsingModulesMixin, CoverageTest):
         omit = twenty
         source = myapp
         source_pkgs = ned
+        source_dirs = cooldir
         plugins =
             plugins.a_plugin
             plugins.another
@@ -596,6 +605,7 @@ class ConfigFileTest(UsingModulesMixin, CoverageTest):
         assert cov.config.concurrency == ["thread"]
         assert cov.config.source == ["myapp"]
         assert cov.config.source_pkgs == ["ned"]
+        assert cov.config.source_dirs == ["cooldir"]
         assert cov.config.disable_warnings == ["abcd", "efgh"]
 
         assert cov.get_exclude_list() == ["if 0:", r"pragma:?\s+no cover", "another_tab"]
