@@ -32,13 +32,13 @@ class JsonReportTest(UsingModulesMixin, CoverageTest):
         self.make_file("a.py", """\
             a = {'b': 1}
             if a.get('a'):
-                b = 1
-            elif a.get('b'):
-                b = 2
-            else:
                 b = 3
+            elif a.get('b'):
+                b = 5
+            else:
+                b = 7
             if not a:
-                b = 4
+                b = 9
             """)
         self._compare_json_reports(cov, expected_result, "a")
 
@@ -83,7 +83,7 @@ class JsonReportTest(UsingModulesMixin, CoverageTest):
         mod = self.start_import_stop(cov, mod_name)
         output_path = os.path.join(self.temp_dir, f"{mod_name}.json")
         cov.json_report(mod, outfile=output_path)
-        with open(output_path) as result_file:
+        with open(output_path, encoding="utf-8") as result_file:
             parsed_result = json.load(result_file)
         self.assert_recent_datetime(
             datetime.strptime(parsed_result['meta']['timestamp'], "%Y-%m-%dT%H:%M:%S.%f"),
@@ -559,3 +559,20 @@ class JsonReportTest(UsingModulesMixin, CoverageTest):
 
     def test_context_relative(self) -> None:
         self.run_context_test(relative_files=True)
+
+    def test_l1_equals_l2(self) -> None:
+        # In results.py, we had a line checking `if l1 == l2` that was never
+        # true.  This test makes it true. The annotations are essential, I
+        # don't know why.
+        self.make_file("wtf.py", """\
+            def function(
+                x: int,
+                y: int,
+            ) -> None:
+                return x + y
+
+            assert function(3, 5) == 8
+            """)
+        cov = coverage.Coverage(branch=True)
+        mod = self.start_import_stop(cov, "wtf")
+        cov.json_report(mod)
