@@ -10,14 +10,14 @@ import io
 import math
 import os.path
 
-from typing import Any
+from typing import Any, Iterable
 from xml.etree import ElementTree
 
 import pytest
 
 import coverage
 from coverage import Coverage
-from coverage.control import Plugins
+from coverage.plugin_support import Plugins
 from coverage.data import line_counts, sorted_lines
 from coverage.exceptions import CoverageWarning, NoSource, PluginError
 from coverage.misc import import_local_file
@@ -53,8 +53,18 @@ class FakeConfig(TPluginConfig):
             return {}
 
 
+def make_plugins(
+    modules: Iterable[str],
+    config: TPluginConfig,
+) -> Plugins:
+    """Construct a Plugins and call plugins.load_from_config() for convenience."""
+    plugins = Plugins()
+    plugins.load_from_config(modules, config)
+    return plugins
+
+
 class LoadPluginsTest(CoverageTest):
-    """Test Plugins.load_plugins directly."""
+    """Test Plugins construction."""
 
     def test_implicit_boolean(self) -> None:
         self.make_file("plugin1.py", """\
@@ -68,10 +78,10 @@ class LoadPluginsTest(CoverageTest):
             """)
 
         config = FakeConfig("plugin1", {})
-        plugins = Plugins.load_plugins([], config)
+        plugins = make_plugins([], config)
         assert not plugins
 
-        plugins = Plugins.load_plugins(["plugin1"], config)
+        plugins = make_plugins(["plugin1"], config)
         assert plugins
 
     def test_importing_and_configuring(self) -> None:
@@ -88,7 +98,7 @@ class LoadPluginsTest(CoverageTest):
             """)
 
         config = FakeConfig("plugin1", {'a': 'hello'})
-        plugins = list(Plugins.load_plugins(["plugin1"], config))
+        plugins = list(make_plugins(["plugin1"], config))
 
         assert len(plugins) == 1
         assert plugins[0].this_is == "me"                   # type: ignore
@@ -119,7 +129,7 @@ class LoadPluginsTest(CoverageTest):
             """)
 
         config = FakeConfig("plugin1", {'a': 'hello'})
-        plugins = list(Plugins.load_plugins(["plugin1", "plugin2"], config))
+        plugins = list(make_plugins(["plugin1", "plugin2"], config))
 
         assert len(plugins) == 2
         assert plugins[0].this_is == "me"                   # type: ignore
@@ -129,7 +139,7 @@ class LoadPluginsTest(CoverageTest):
 
         # The order matters...
         config = FakeConfig("plugin1", {'a': 'second'})
-        plugins = list(Plugins.load_plugins(["plugin2", "plugin1"], config))
+        plugins = list(make_plugins(["plugin2", "plugin1"], config))
 
         assert len(plugins) == 2
         assert plugins[0].options == {}                     # type: ignore
@@ -138,7 +148,7 @@ class LoadPluginsTest(CoverageTest):
 
     def test_cant_import(self) -> None:
         with pytest.raises(ImportError, match="No module named '?plugin_not_there'?"):
-            _ = Plugins.load_plugins(["plugin_not_there"], NullConfig())
+            _ = make_plugins(["plugin_not_there"], NullConfig())
 
     def test_plugin_must_define_coverage_init(self) -> None:
         self.make_file("no_plugin.py", """\
@@ -147,7 +157,7 @@ class LoadPluginsTest(CoverageTest):
             """)
         msg_pat = "Plugin module 'no_plugin' didn't define a coverage_init function"
         with pytest.raises(PluginError, match=msg_pat):
-            list(Plugins.load_plugins(["no_plugin"], NullConfig()))
+            list(make_plugins(["no_plugin"], NullConfig()))
 
 
 class PluginTest(CoverageTest):
@@ -161,7 +171,7 @@ class PluginTest(CoverageTest):
                 pass
             def coverage_init(reg, options):
                 reg.add_noop(Plugin())
-            with open("evidence.out", "w") as f:
+            with open("evidence.out", "w", encoding="utf-8") as f:
                 f.write("we are here!")
             """)
 
@@ -171,7 +181,7 @@ class PluginTest(CoverageTest):
         cov.start()
         cov.stop()      # pragma: nested
 
-        with open("evidence.out") as f:
+        with open("evidence.out", encoding="utf-8") as f:
             assert f.read() == "we are here!"
 
     def test_missing_plugin_raises_import_error(self) -> None:
@@ -272,6 +282,18 @@ class PluginTest(CoverageTest):
         out = self.run_command("coverage html -q")  # sneak in a test of -q
         assert out == ""
 
+    def test_coverage_init_plugins(self) -> None:
+        called = False
+        def coverage_init(reg: Plugins) -> None:    # pylint: disable=unused-argument
+            nonlocal called
+            called = True
+
+        cov = coverage.Coverage(plugins=[coverage_init])
+        # Calls _init() and loads plugins
+        cov.sys_info()
+
+        assert called
+
 
 @pytest.mark.skipif(testenv.PLUGINS, reason="This core doesn't support plugins.")
 class PluginWarningOnPyTracerTest(CoverageTest):
