@@ -5,6 +5,7 @@
 
 from __future__ import annotations
 
+import csv
 import glob
 import os
 import os.path
@@ -14,6 +15,7 @@ import stat
 import sys
 import textwrap
 
+from pathlib import Path
 from typing import Any
 
 import pytest
@@ -23,8 +25,9 @@ from coverage import env
 from coverage.data import line_counts
 from coverage.files import abs_file, python_reported_file
 
+from tests import testenv
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import re_lines_text
+from tests.helpers import re_line, re_lines, re_lines_text
 
 
 class ProcessTest(CoverageTest):
@@ -50,6 +53,7 @@ class ProcessTest(CoverageTest):
             """)
 
         self.assert_doesnt_exist(".coverage")
+        self.add_test_modules_to_pythonpath()
         out = self.run_command("coverage run mycode.py")
         self.assert_exists(".coverage")
         assert out == 'done\n'
@@ -58,7 +62,7 @@ class ProcessTest(CoverageTest):
         # Test that we are setting COVERAGE_RUN when we run.
         self.make_file("envornot.py", """\
             import os
-            print(os.environ.get("COVERAGE_RUN", "nope"))
+            print(os.getenv("COVERAGE_RUN", "nope"))
             """)
         self.del_environ("COVERAGE_RUN")
         # Regular Python doesn't have the environment variable.
@@ -363,44 +367,59 @@ class ProcessTest(CoverageTest):
         self.make_file("fork.py", """\
             import os
 
-            def child():
-                print('Child!')
+            print(f"parent,{os.getpid()}", flush=True)
+            ret = os.fork()
 
-            def main():
-                ret = os.fork()
-
-                if ret == 0:
-                    child()
-                else:
-                    os.waitpid(ret, 0)
-
-            main()
+            if ret == 0:
+                print(f"child,{os.getpid()}", flush=True)
+            else:
+                os.waitpid(ret, 0)
             """)
+        total_lines = 6
 
-        out = self.run_command("coverage run -p fork.py")
-        assert out == 'Child!\n'
+        self.set_environ("COVERAGE_DEBUG_FILE", "debug.out")
+        out = self.run_command("coverage run --debug=pid,process,trace -p fork.py")
+        pids = {key:int(pid) for key, pid in csv.reader(out.splitlines())}
+        assert set(pids) == {"parent", "child"}
         self.assert_doesnt_exist(".coverage")
 
         # After running the forking program, there should be two
-        # .coverage.machine.123 files.
+        # .coverage.machine.pid.randomword files.  The pids should match our
+        # processes, and the files should have different random words at the
+        # end of the file name.
         self.assert_file_count(".coverage.*", 2)
-
-        # The two data files should have different random numbers at the end of
-        # the file name.
         data_files = glob.glob(".coverage.*")
-        nums = {name.rpartition(".")[-1] for name in data_files}
-        assert len(nums) == 2, f"Same random: {data_files}"
+        filepids = {int(name.split(".")[-2]) for name in data_files}
+        assert filepids == set(pids.values())
+        suffixes = {name.split(".")[-1] for name in data_files}
+        assert len(suffixes) == 2, f"Same random suffix: {data_files}"
 
-        # Combine the parallel coverage data files into .coverage .
+        # Each data file should have a subset of the lines.
+        for data_file in data_files:
+            data = coverage.CoverageData(data_file)
+            data.read()
+            assert line_counts(data)["fork.py"] < total_lines
+
+        # Combine the parallel coverage data files into a .coverage file.
+        # After combining, there should be only the .coverage file.
         self.run_command("coverage combine")
         self.assert_exists(".coverage")
-
-        # After combining, there should be only the .coverage file.
         self.assert_file_count(".coverage.*", 0)
 
         data = coverage.CoverageData()
         data.read()
-        assert line_counts(data)['fork.py'] == 9
+        assert line_counts(data)["fork.py"] == total_lines
+
+        debug_text = Path("debug.out").read_text(encoding="utf-8")
+        ppid = pids["parent"]
+        cpid = pids["child"]
+        assert ppid != cpid
+        plines = re_lines(fr"{ppid}\.[0-9a-f]+: New process: pid={ppid}, executable", debug_text)
+        assert len(plines) == 1
+        clines = re_lines(fr"{cpid}\.[0-9a-f]+: New process: forked {ppid} -> {cpid}", debug_text)
+        assert len(clines) == 1
+        reported_pids = {line.split(".")[0] for line in debug_text.splitlines()}
+        assert len(reported_pids) == 2
 
     def test_warnings_during_reporting(self) -> None:
         # While fixing issue #224, the warnings were being printed far too
@@ -462,6 +481,7 @@ class ProcessTest(CoverageTest):
         assert "Hello\n" in out
         assert "warning" not in out
 
+    @pytest.mark.skipif(env.METACOV, reason="Can't test tracers changing during metacoverage")
     def test_warning_trace_function_changed(self) -> None:
         self.make_file("settrace.py", """\
             import sys
@@ -492,24 +512,21 @@ class ProcessTest(CoverageTest):
             # Show the current frame's trace function, so that we can test what the
             # command-line options do to the trace function used.
 
-            import sys
+            import inspect
 
             # Show what the trace function is.  If a C-based function is used, then f_trace
             # may be None.
-            trace_fn = sys._getframe(0).f_trace
+            trace_fn = inspect.currentframe().f_trace
             if trace_fn is None:
                 trace_name = "None"
             else:
-                # Get the name of the tracer class.  Py3k has a different way to get it.
+                # Get the name of the tracer class.
                 try:
-                    trace_name = trace_fn.im_class.__name__
+                    trace_name = trace_fn.__self__.__class__.__name__
                 except AttributeError:
-                    try:
-                        trace_name = trace_fn.__self__.__class__.__name__
-                    except AttributeError:
-                        # A C-based function could also manifest as an f_trace value
-                        # which doesn't have im_class or __self__.
-                        trace_name = trace_fn.__class__.__name__
+                    # A C-based function could also manifest as an f_trace value
+                    # which doesn't have __self__.
+                    trace_name = trace_fn.__class__.__name__
 
             print(trace_name)
             """)
@@ -519,10 +536,12 @@ class ProcessTest(CoverageTest):
         assert py_out == "None\n"
 
         cov_out = self.run_command("coverage run showtrace.py")
-        if os.environ.get('COVERAGE_TEST_TRACER', 'c') == 'c':
+        if testenv.C_TRACER:
             # If the C trace function is being tested, then regular running should have
             # the C function, which registers itself as f_trace.
             assert cov_out == "CTracer\n"
+        elif testenv.SYS_MON:
+            assert cov_out == "None\n"
         else:
             # If the Python trace function is being tested, then regular running will
             # also show the Python function.
@@ -556,30 +575,6 @@ class ProcessTest(CoverageTest):
         )
         assert msg in out
 
-    @pytest.mark.expensive
-    @pytest.mark.skipif(not env.C_TRACER, reason="fullcoverage only works with the C tracer.")
-    @pytest.mark.skipif(env.METACOV, reason="Can't test fullcoverage when measuring ourselves")
-    def test_fullcoverage(self) -> None:
-        # fullcoverage is a trick to get stdlib modules measured from
-        # the very beginning of the process. Here we import os and
-        # then check how many lines are measured.
-        self.make_file("getenv.py", """\
-            import os
-            print("FOOEY == %s" % os.getenv("FOOEY"))
-            """)
-
-        fullcov = os.path.join(os.path.dirname(coverage.__file__), "fullcoverage")
-        self.set_environ("FOOEY", "BOO")
-        self.set_environ("PYTHONPATH", fullcov)
-        out = self.run_command("python -X frozen_modules=off -m coverage run -L getenv.py")
-        assert out == "FOOEY == BOO\n"
-        data = coverage.CoverageData()
-        data.read()
-        # The actual number of executed lines in os.py when it's
-        # imported is 120 or so.  Just running os.getenv executes
-        # about 5.
-        assert line_counts(data)['os.py'] > 50
-
     # Pypy passes locally, but fails in CI? Perhaps the version of macOS is
     # significant?  https://foss.heptapod.net/pypy/pypy/-/issues/3074
     @pytest.mark.skipif(env.PYPY, reason="PyPy is unreliable with this test")
@@ -611,7 +606,7 @@ class ProcessTest(CoverageTest):
             """)
 
         # Some of our testing infrastructure can issue warnings.
-        # Turn it all off for the sub-process.
+        # Turn it all off for the subprocess.
         self.del_environ("COVERAGE_TESTING")
 
         out = self.run_command("python allok.py")
@@ -666,36 +661,44 @@ class EnvironmentTest(CoverageTest):
     def assert_tryexecfile_output(self, expected: str, actual: str) -> None:
         """Assert that the output we got is a successful run of try_execfile.py.
 
-        `expected` and `actual` must be the same, modulo a few slight known
-        platform differences.
+        `expected` and `actual` must be the same.
 
         """
         # First, is this even credible try_execfile.py output?
         assert '"DATA": "xyzzy"' in actual
+        # If this fails, "+" is actual, and "-" is expected
         assert actual == expected
 
     def test_coverage_run_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         actual = self.run_command("coverage run run_me.py")
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_far_away_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/overthere/prog.py", f.read())
         expected = self.run_command("python sub/overthere/prog.py")
         actual = self.run_command("coverage run sub/overthere/prog.py")
         self.assert_tryexecfile_output(expected, actual)
 
+    @pytest.mark.skipif(not env.WINDOWS, reason="This is about Windows paths")
+    def test_coverage_run_far_away_is_like_python_windows(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("sub/overthere/prog.py", f.read())
+        expected = self.run_command("python sub\\overthere\\prog.py")
+        actual = self.run_command("coverage run sub\\overthere\\prog.py")
+        self.assert_tryexecfile_output(expected, actual)
+
     def test_coverage_run_dashm_is_like_python_dashm(self) -> None:
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command("coverage run -m process_test.try_execfile")
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dir_is_like_python_dir(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python with_main")
@@ -703,7 +706,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_no_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python -m with_main")
@@ -711,7 +714,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_with_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
         self.make_file("with_main/__init__.py", "")
 
@@ -725,10 +728,10 @@ class EnvironmentTest(CoverageTest):
         When imported by -m, a module's __name__ is __main__, but we need the
         --source machinery to know and respect the original name.
         """
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command(
-            "coverage run --source process_test.try_execfile -m process_test.try_execfile"
+            "coverage run --source process_test.try_execfile -m process_test.try_execfile",
         )
         self.assert_tryexecfile_output(expected, actual)
 
@@ -745,10 +748,10 @@ class EnvironmentTest(CoverageTest):
             [run]
             disable_warnings = module-not-measured
             """)
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command(
-            "coverage run --source process_test -m process_test.try_execfile"
+            "coverage run --source process_test -m process_test.try_execfile",
         )
         self.assert_tryexecfile_output(expected, actual)
 
@@ -767,6 +770,7 @@ class EnvironmentTest(CoverageTest):
             import process_test.try_execfile
             """)
 
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python myscript")
         actual = self.run_command("coverage run --source process_test myscript")
         self.assert_tryexecfile_output(expected, actual)
@@ -778,7 +782,7 @@ class EnvironmentTest(CoverageTest):
     def test_coverage_run_dashm_is_like_python_dashm_off_path(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/242
         self.make_file("sub/__init__.py", "")
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/run_me.py", f.read())
 
         expected = self.run_command("python -m sub.run_me")
@@ -797,13 +801,46 @@ class EnvironmentTest(CoverageTest):
         # Test running coverage from a zip file itself.  Some environments
         # (windows?) zip up the coverage main to be used as the coverage
         # command.
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         cov_main = os.path.join(TESTS_DIR, "covmain.zip")
         actual = self.run_command(f"python {cov_main} run run_me.py")
         self.assert_tryexecfile_output(expected, actual)
 
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    @pytest.mark.skipif(
+        env.WINDOWS,
+        reason="Windows gets this wrong: https://github.com/python/cpython/issues/131484",
+    )
+    def test_pythonsafepath(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("run_me.py", f.read())
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python run_me.py")
+        actual = self.run_command("coverage run run_me.py")
+        self.assert_tryexecfile_output(expected, actual)
+
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    def test_pythonsafepath_dashm_runme(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("run_me.py", f.read())
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python run_me.py")
+        actual = self.run_command("python -m coverage run run_me.py")
+        self.assert_tryexecfile_output(expected, actual)
+
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    def test_pythonsafepath_dashm(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("with_main/__main__.py", f.read())
+
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python -m with_main")
+        actual = self.run_command("coverage run -m with_main")
+        assert re.search("No module named '?with_main'?", actual)
+        assert re.search("No module named '?with_main'?", expected)
+
     def test_coverage_custom_script(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/678
         # If sys.path[0] isn't the Python default, then coverage.py won't
@@ -812,7 +849,7 @@ class EnvironmentTest(CoverageTest):
             SOMETHING = "hello-xyzzy"
             """)
         abc = os.path.abspath("a/b/c")
-        self.make_file("run_coverage.py", """\
+        self.make_file("run_coverage.py", f"""\
             import sys
             sys.path[0:0] = [
                 r'{abc}',
@@ -823,7 +860,7 @@ class EnvironmentTest(CoverageTest):
 
             if __name__ == '__main__':
                 sys.exit(coverage.cmdline.main())
-            """.format(abc=abc))
+            """)
         self.make_file("how_is_it.py", """\
             import pprint, sys
             pprint.pprint(sys.path)
@@ -840,18 +877,21 @@ class EnvironmentTest(CoverageTest):
     @pytest.mark.skipif(env.WINDOWS, reason="Windows can't make symlinks")
     @pytest.mark.skipif(
         platform.python_version().endswith("+"),
-        reason="setuptools barfs on dev versions: https://github.com/pypa/packaging/issues/678"
+        reason="setuptools barfs on dev versions: https://github.com/pypa/packaging/issues/678",
         # https://github.com/nedbat/coveragepy/issues/1556
-        # TODO: get rid of pkg_resources
     )
     def test_bug_862(self) -> None:
-        # This simulates how pyenv and pyenv-virtualenv end up creating the
-        # coverage executable.
-        self.make_file("elsewhere/bin/fake-coverage", """\
-            #!{executable}
-            import sys, pkg_resources
-            sys.exit(pkg_resources.load_entry_point('coverage', 'console_scripts', 'coverage')())
-            """.format(executable=sys.executable))
+        # This used to simulate how pyenv and pyenv-virtualenv create the
+        # coverage executable.  Now the code shows how venv does it.
+        self.make_file("elsewhere/bin/fake-coverage", f"""\
+            #!{sys.executable}
+            import re
+            import sys
+            from coverage.cmdline import main
+            if __name__ == '__main__':
+                sys.argv[0] = re.sub(r'(-script\\.pyw|\\.exe)?$', '', sys.argv[0])
+                sys.exit(main())
+            """)
         os.chmod("elsewhere/bin/fake-coverage", stat.S_IREAD | stat.S_IEXEC)
         os.symlink("elsewhere", "somewhere")
         self.make_file("foo.py", "print('inside foo')")
@@ -918,10 +958,13 @@ class ExcepthookTest(CoverageTest):
         # executed.
         data = coverage.CoverageData()
         data.read()
+        print(f"{line_counts(data) = }")
+        print(f"{data = }")
+        print("data.lines excepthook.py:", data.lines(os.path.abspath('excepthook.py')))
         assert line_counts(data)['excepthook.py'] == 7
 
     @pytest.mark.skipif(not env.CPYTHON,
-        reason="non-CPython handles excepthook exits differently, punt for now."
+        reason="non-CPython handles excepthook exits differently, punt for now.",
     )
     def test_excepthook_exit(self) -> None:
         self.make_file("excepthook_exit.py", """\
@@ -980,8 +1023,8 @@ class AliasedCommandTest(CoverageTest):
 
     def test_wrong_alias_doesnt_work(self) -> None:
         # "coverage2" doesn't work on py3
-        assert sys.version_info[0] in [2, 3]    # Let us know when Python 4 is out...
-        badcmd = "coverage%d" % (5 - sys.version_info[0])
+        assert sys.version_info[0] == 3    # Let us know when Python 4 is out...
+        badcmd = "coverage2"
         out = self.run_command(badcmd)
         assert "Code coverage for Python" not in out
 
@@ -1067,7 +1110,7 @@ class FailUnderTest(CoverageTest):
             "a = 1\n" +
             "b = 2\n" * 2000 +
             "if a > 3:\n" +
-            "    c = 4\n"
+            "    c = 4\n",
         )
         self.make_data_file(lines={abs_file("ninety_nine_plus.py"): range(1, 2002)})
         st, out = self.run_command_status("coverage report --fail-under=100")
@@ -1076,6 +1119,72 @@ class FailUnderTest(CoverageTest):
         assert expected == self.last_line_squeezed(out)
 
 
+class CoverageCoreTest(CoverageTest):
+    """Test that cores are chosen correctly."""
+    # This doesn't test failure modes, only successful requests.
+    try:
+        from coverage.tracer import CTracer
+        has_ctracer = True
+    except ImportError:
+        has_ctracer = False
+
+    def test_core_default(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.del_environ("COVERAGE_CORE")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        # if env.PYBEHAVIOR.pep669:
+        #     assert core == "core: SysMonitor"
+        if self.has_ctracer:
+            assert core == "core: CTracer"
+        else:
+            assert core == "core: PyTracer"
+
+    @pytest.mark.skipif(not has_ctracer, reason="No CTracer to request")
+    def test_core_request_ctrace(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "ctrace")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: CTracer"
+
+    def test_core_request_pytrace(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "pytrace")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: PyTracer"
+
+    def test_core_request_sysmon(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "sysmon")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        warns = re_lines(r"CoverageWarning: sys.monitoring isn't available", out)
+        if env.PYBEHAVIOR.pep669:
+            assert core == "core: SysMonitor"
+            assert not warns
+        else:
+            assert core in ("core: CTracer", "core: PyTracer")
+            assert warns
+
+    def test_core_request_nosuchcore(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "nosuchcore")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run numbers.py")
+        assert "Unknown core value: 'nosuchcore'\n" in out
+        assert "123 456" not in out
+
+
 class FailUnderNoFilesTest(CoverageTest):
     """Test that nothing to report results in an error exit status."""
     def test_report(self) -> None:
@@ -1132,9 +1241,9 @@ class YankedDirectoryTest(CoverageTest):
         assert all(line in out for line in lines)
 
 
-@pytest.mark.skipif(env.METACOV, reason="Can't test sub-process pth file during metacoverage")
+@pytest.mark.skipif(env.METACOV, reason="Can't test subprocess pth file during metacoverage")
 class ProcessStartupTest(CoverageTest):
-    """Test that we can measure coverage in sub-processes."""
+    """Test that we can measure coverage in subprocesses."""
 
     def setUp(self) -> None:
         super().setUp()
@@ -1147,7 +1256,7 @@ class ProcessStartupTest(CoverageTest):
             """)
         # sub.py will write a few lines.
         self.make_file("sub.py", """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!\\n")
             f.close()
             """)
@@ -1167,7 +1276,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         import main             # pylint: disable=unused-import, import-error
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         # Read the data from .coverage
@@ -1186,7 +1295,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         self.run_command("coverage run main.py")
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         self.run_command("coverage combine")
@@ -1259,7 +1368,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
             self.make_file(path("__init__.py"), "")
         # sub.py will write a few lines.
         self.make_file(path("sub.py"), """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!")
             f.close()
             """)
@@ -1277,7 +1386,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
 
         self.run_command(cmd)
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!"
 
         # Read the data from .coverage
